import os
import random
from practicejapanese.quizzes import audio_quiz, vocab_quiz, kanji_quiz

def random_quiz():
    from practicejapanese.core.vocab import load_vocab
    from practicejapanese.core.kanji import load_kanji

    vocab_path = os.path.abspath(os.path.join(os.path.dirname(__file__), "../data/N5Vocab.csv"))
    kanji_path = os.path.abspath(os.path.join(os.path.dirname(__file__), "../data/N5Kanji.csv"))

    vocab_list = load_vocab(vocab_path)
    kanji_list = load_kanji(kanji_path)

    from practicejapanese.core.utils import lowest_score_items
    from practicejapanese.quizzes import filling_quiz
    quizzes = [
        ("Vocab Quiz", lambda: vocab_quiz.ask_question(lowest_score_items(vocab_path, vocab_list, score_col=3))),
        ("Kanji Quiz", lambda: kanji_quiz.ask_question(lowest_score_items(kanji_path, kanji_list, score_col=3))),
        ("Kanji Fill-in Quiz", lambda: filling_quiz.ask_question(lowest_score_items(vocab_path, vocab_list, score_col=4))),
        ("Audio Quiz", lambda: audio_quiz.ask_question(lowest_score_items(vocab_path, vocab_list, score_col=4)))
    ]
    import threading
    import queue
    def preload_question(q):
        while True:
            selected_name, selected_quiz = random.choice(quizzes)
            q.put((selected_name, selected_quiz))

    q = queue.Queue(maxsize=3)
    loader_thread = threading.Thread(target=preload_question, args=(q,), daemon=True)
    loader_thread.start()
    try:
        while True:
            selected_name, selected_quiz = q.get()  # Wait for next question to be ready
            print(f"Selected: {selected_name}")
            selected_quiz()
            print()  # Add empty line after each question
    except KeyboardInterrupt:
        print("\nQuiz interrupted. Goodbye!")



import os
import random
import requests
from functools import lru_cache
from practicejapanese.core.vocab import load_vocab
from practicejapanese.core.utils import quiz_loop, update_score, lowest_score_items

CSV_PATH = os.path.abspath(os.path.join(
    os.path.dirname(__file__), "..", "data", "N5Vocab.csv"))


def ask_question(vocab_list):
    """Ask the user to replace hiragana with the correct kanji."""
    word = random.choice(vocab_list)
    questions = generate_questions(word)
    if not questions:
        print(f"Attempted to generate questions for: {word}")
        print("No fill-in questions generated. Check API or vocab data.")
        return
    # Select two distinct questions for context
    if len(questions) >= 2:
        selected = random.sample(questions, 2)
    else:
        selected = [questions[0]]
    print("Replace the highlighted hiragana with the correct kanji:")
    for idx, (sentence, answer) in enumerate(selected):
        print(f"{sentence}")
    # Use the first question's answer for checking
    answer = selected[0][1]
    user_input = input("Your answer (kanji): ").strip()
    correct = (user_input == answer)
    if correct:
        print("Correct!")
    else:
        print(f"Wrong. Correct kanji: {answer}")
    update_score(CSV_PATH, answer, correct, score_col=4)
    print()


def run():
    def dynamic_quiz_loop():
        try:
            while True:
                vocab_list = load_vocab(CSV_PATH)
                lowest_vocab = lowest_score_items(
                    CSV_PATH, vocab_list, score_col=4)
                if not lowest_vocab:
                    print("No vocab found.")
                    return
                ask_question(lowest_vocab)
        except KeyboardInterrupt:
            print("\nExiting quiz. Goodbye!")
    dynamic_quiz_loop()


@lru_cache(maxsize=128)
def cached_fetch_sentences(reading, kanji, limit=5):
    url = f"https://tatoeba.org/en/api_v0/search?from=jpn&query={reading}&limit={limit}"
    try:
        resp = requests.get(url)
        data = resp.json()
    except Exception:
        return tuple()
    sentences = []
    for item in data.get("results", []):
        text = item.get("text", "")
        if reading in text or kanji in text:
            sentences.append(text)
    return tuple(sentences)


def generate_questions(vocab_list):
    questions = []
    reading, kanji = vocab_list[1], vocab_list[0]
    sentences = cached_fetch_sentences(reading, kanji, 5)
    for sentence in sentences:
        if kanji in sentence:
            formatted = sentence.replace(kanji, f"[{reading}]")
            questions.append((formatted, kanji))
    return questions


if __name__ == "__main__":
    print("Running Kanji Fill-in Quiz in DEV mode...")
    run()


import sys
from practicejapanese import __version__ as VERSION
from practicejapanese.quizzes import audio_quiz, vocab_quiz, kanji_quiz
from practicejapanese.core.quiz_runner import random_quiz
from practicejapanese.core.dev_mode import run_dev_mode
import os

def main():
    if len(sys.argv) > 1:
        if sys.argv[1] == "-v":
            print(f"PracticeJapanese version {VERSION}")
            return
        elif sys.argv[1] == "-dev":
            run_dev_mode()
            return

    print("Select quiz type:")
    print("1. Random Quiz (random category each time)")
    print("2. Vocab Quiz")
    print("3. Kanji Quiz")
    print("4. Kanji Fill-in Quiz")
    print("5. Audio Quiz")
    print("6. Reset all scores to zero")
    choice = input("Enter number: ").strip()
    try:
        if choice == "1":
            random_quiz()
        elif choice == "2":
            vocab_quiz.run()
            print()  # Add empty line after each question
        elif choice == "3":
            kanji_quiz.run()
            print()  # Add empty line after each question
        elif choice == "4":
            from practicejapanese.quizzes import filling_quiz
            filling_quiz.run()
            print()  # Add empty line after each question
        elif choice == "5":
            audio_quiz.run()
            print()  # Add empty line after each question
        elif choice == "6":
            from practicejapanese.core.utils import reset_scores
            reset_scores()
        else:
            print("Invalid choice.")
    except KeyboardInterrupt:
        print("\nQuiz interrupted. Goodbye!")

if __name__ == "__main__":
    main()


from practicejapanese.core.vocab import load_vocab
from practicejapanese.core.utils import quiz_loop, update_score, lowest_score_items
import random
import os

CSV_PATH = os.path.abspath(os.path.join(os.path.dirname(__file__), "..", "data", "N5Vocab.csv"))

def ask_question(vocab_list):
    item = random.choice(vocab_list)
    print()  # Add empty line before the question
    correct = False
    if random.choice([True, False]):
        print(f"Reading: {item[1]}")
        print(f"Meaning: {item[2]}")
        answer = input("What is the Kanji? ")
        correct = (answer == item[0])
        if correct:
            print("Correct!")
        else:
            print(f"Incorrect. The correct Kanji is: {item[0]}")
    else:
        print(f"Kanji: {item[0]}")
        print(f"Meaning: {item[2]}")
        answer = input("What is the Reading? ")
        correct = (answer == item[1])
        if correct:
            print("Correct!")
        else:
            print(f"Incorrect. The correct Reading is: {item[1]}")
    update_score(CSV_PATH, item[0], correct, score_col=3)
    print()  # Add empty line after the question

def run():
    def dynamic_quiz_loop():
        try:
            while True:
                vocab_list = load_vocab(CSV_PATH)
                lowest_vocab = lowest_score_items(CSV_PATH, vocab_list, score_col=3)
                if not lowest_vocab:
                    print("No vocab found.")
                    return
                ask_question(lowest_vocab)
        except KeyboardInterrupt:
            print("\nExiting quiz. Goodbye!")
    dynamic_quiz_loop()


import random
import os
import csv


def reset_scores():
    print("Resetting all scores to zero...")
    for csv_path in [
        os.path.abspath(os.path.join(os.path.dirname(
            __file__), "..", "data", "N5Kanji.csv")),
        os.path.abspath(os.path.join(os.path.dirname(
            __file__), "..", "data", "N5Vocab.csv")),
    ]:
        temp_path = csv_path + '.temp'
        updated_rows = []
        with open(csv_path, 'r', encoding='utf-8') as infile:
            reader = csv.reader(infile)
            for row in reader:
                if row:
                    if os.path.basename(csv_path) == "N5Vocab.csv":
                        # Last two columns are scores
                        if len(row) >= 2:
                            row[-1] = '0'
                            row[-2] = '0'
                    else:
                        # Only last column is score
                        row[-1] = '0'
                updated_rows.append(row)
        with open(temp_path, 'w', encoding='utf-8', newline='') as outfile:
            writer = csv.writer(outfile)
            writer.writerows(updated_rows)
        os.replace(temp_path, csv_path)
    print("All scores reset to zero.")


def quiz_loop(quiz_func, data):
    try:
        while True:
            quiz_func(data)
    except KeyboardInterrupt:
        print("\nExiting quiz. Goodbye!")


# --- DRY helpers for quizzes ---


def update_score(csv_path, key, correct, score_col=-1):
    """
    Update the score for a given key in the specified column (score_col).
    If correct, increment; else, set to zero.
    """
    temp_path = csv_path + '.temp'
    updated_rows = []
    with open(csv_path, 'r', encoding='utf-8') as infile:
        reader = csv.reader(infile)
        for row in reader:
            if row and row[0] == key:
                if correct:
                    try:
                        row[score_col] = str(int(row[score_col]) + 1)
                    except (ValueError, IndexError):
                        row[score_col] = '1'
                else:
                    row[score_col] = '0'
            updated_rows.append(row)
    with open(temp_path, 'w', encoding='utf-8', newline='') as outfile:
        writer = csv.writer(outfile)
        writer.writerows(updated_rows)
    os.replace(temp_path, csv_path)


def lowest_score_items(csv_path, vocab_list, score_col):
    """
    Returns items from vocab_list whose key (row[0]) has the lowest score in score_col.
    """
    with open(csv_path, encoding="utf-8") as f:
        reader = csv.reader(f)
        scores = [(row[0], int(row[score_col]) if len(row) > score_col and row[score_col].isdigit() else 0)
                  for row in reader if row and row[0]]
    if not scores:
        return []
    min_score = min(score for _, score in scores)
    lowest_keys = [k for k, s in scores if s == min_score]
    return [item for item in vocab_list if item[0] in lowest_keys]



import csv

def load_vocab(path):
    vocab_list = []
    with open(path, encoding="utf-8") as f:
        reader = csv.reader(f)
        for row in reader:
            if len(row) < 5:
                continue
            if row[0].strip():
                # word, reading, meaning, vocab_score, filling_score
                vocab_list.append((row[0].strip(), row[1].strip(), row[2].strip(), row[3].strip(), row[4].strip()))
    return vocab_list


import sys
import os
from practicejapanese import __version__ as VERSION

def run_dev_mode():
    print("Developer mode activated!")
    print(f"Python version: {sys.version}")
    print(f"Current working directory: {os.getcwd()}")
    print(f"Available quizzes: vocab_quiz, kanji_quiz, filling_quiz")
    print("Dev options:")
    print("1. Show all scores")
    print("2. Exit dev mode")
    dev_choice = input("Enter dev option: ").strip()
    if dev_choice == "1":
        from practicejapanese.core.vocab import load_vocab
        from practicejapanese.core.kanji import load_kanji
        vocab_path = os.path.abspath(os.path.join(os.path.dirname(__file__), "../data/N5Vocab.csv"))
        kanji_path = os.path.abspath(os.path.join(os.path.dirname(__file__), "../data/N5Kanji.csv"))
        vocab_list = load_vocab(vocab_path)
        kanji_list = load_kanji(kanji_path)
        print("\nKanji Scores:")
        for entry in kanji_list:
            if isinstance(entry, tuple) and len(entry) >= 4:
                kanji = entry[0]
                score = entry[-1]
                print(f"{kanji}: {score}")
        print("\nAll scores displayed.")
        print("\nVocab Scores:")
        for entry in vocab_list:
            if isinstance(entry, tuple) and len(entry) >= 5:
                word = entry[0]
                vocab_score = entry[3]
                filling_score = entry[4]
                print(f"{word}: Vocab Quiz Score = {vocab_score}, Filling Quiz Score = {filling_score}")
    else:
        print("Exiting dev mode.")



import csv

def load_kanji(path):
    kanji_list = []
    with open(path, encoding="utf-8") as f:
        reader = csv.reader(f)
        for row in reader:
            if len(row) < 4:
                continue
            # Only take kanji, readings, meaning, score
            kanji_list.append((row[0].strip(), row[1].strip(), row[2].strip(), row[3].strip()))
    return kanji_list


from setuptools import setup, find_packages
import os
import re

def read_version(): 
    init_path = os.path.join(os.path.dirname(__file__), "practicejapanese", "__init__.py")
    with open(init_path, "r") as f:
        content = f.read()
    match = re.search(r'__version__\s*=\s*[\'"]([^\'"]+)[\'"]', content)
    return match.group(1) if match else "0.0.0"

def read_requirements():
    req_path = os.path.join(os.path.dirname(__file__), "requirements.txt")
    with open(req_path, "r") as f:
        lines = f.readlines()
    return [line.strip() for line in lines if line.strip() and not line.startswith('#')]

setup(
    name="pjapp",
    version=read_version(),
    packages=find_packages(),
    install_requires=read_requirements(),
    entry_points={
        "console_scripts": [
            "pjapp=practicejapanese.main:main"
        ]
    },
    include_package_data=True,
)